{$I DacDemo.inc}
unit PgDacDemoForm;

interface

uses
  SysUtils, Classes, DB, Variants,
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, Forms, PgDacVcl,
  Dialogs, Menus, ImgList, StdCtrls, ComCtrls, Buttons, ExtCtrls, ToolWin,
  PgDACAbout, DemoFrame, DemoForm, DemoBase, DBAccess, DAScript,
  PgScript, PgAccess;

type
  TPgDacForm = class(TDemoForm)
    PgConnection: TPgConnection;
    PgConnectDialog1: TPgConnectDialog;
    scCreate: TPgScript;
    scDrop: TPgScript;
    procedure lbAboutClick(Sender: TObject); override;
    procedure cbDebugClick(Sender: TObject);
  private
    { Private declarations }
  protected
    //Product customization
    function GetConnection: TCustomDAConnection; override;
    function ApplicationTitle: string; override;
    function ProductName: string; override;
    procedure RegisterDemos; override;
  public
    function ProductColor: TColor; override;
    procedure ExecCreateScript; override;
    procedure ExecDropScript; override;
  end;

var
  PgDacForm: TPgDacForm;

implementation

uses
{$IFDEF CRDBGRID}
{$IFNDEF FPC}
  CRDBGrid,
{$ENDIF}
{$ENDIF}
{$IFNDEF FPC}
  Pictures,
{$IFNDEF STD}
  Alerter,
{$ENDIF}
{$ENDIF}
{$IFNDEF STD}
  Dump, Loader,
{$ENDIF}
  CachedUpdates, ConnectDialog, MasterDetail, Query, Sql, StoredProc,
  FilterAndIndex, Table, Text, UpdateSQL, VTable, Lock, AggregateAndCalc;

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

function TPgDacForm.GetConnection: TCustomDAConnection;
begin
  Result := PgConnection;
end;

function TPgDacForm.ProductColor: TColor;
begin
  Result := $006E5200;
end;

procedure TPgDacForm.ExecCreateScript;
begin
  scCreate.OnError := OnScriptError;
  scCreate.Execute;
end;

procedure TPgDacForm.ExecDropScript;
begin
  scDrop.OnError := OnScriptError;
  scDrop.Execute;
end;

function TPgDacForm.ApplicationTitle: string;
begin
  Result := 'PostgreSQL Data Access Components demos';
end;

function TPgDacForm.ProductName: string;
begin
  Result := 'PgDAC';
end;

procedure TPgDacForm.RegisterDemos;
begin
  Demos.RegisterCategory('PgDAC Demo', 'PgDAC Demo');
  Demos.RegisterCategory('Working with components', 'Working with components');
  Demos.RegisterCategory('General demos', 'General demos');
  Demos.RegisterCategory('PgSQL-specific', 'PgSQL-specific');

{$IFNDEF STD}
{$IFNDEF FPC}
  Demos.RegisterDemo('Alerter', 'Using TPgAlerter component',
    'Uses the TPgAlerter component to send notifications between connections',
    'Working with components', TAlerterFrame, 12);
{$ENDIF}
{$ENDIF}
  Demos.RegisterDemo('ConnectDialog', 'Customizing login dialog',
    'Demonstrates how to customize the PgDAC connect dialog. Changes the standard ' +
    'PgDAC connect dialog to two custom connect dialogs. ' +
    'The first customized sample dialog is inherited from the TForm class, ' +
    'and the second one is inherited from the default PgDAC connect dialog class.',
    'Working with components', TConnectDialogFrame, 2);
{$IFDEF CRDBGRID}
{$IFNDEF FPC}
  Demos.RegisterDemo('CRDBGrid', 'Using TCRDBGrid component',
    'Demonstrates how to work with the TCRDBGrid component.  Shows off the main ' +
    'TCRDBGrid features, like filtering, searching, stretching, using compound headers, and more.',
    'Working with components', TCRDBGridFrame, 3);
{$ENDIF}
{$ENDIF}
{$IFNDEF STD}
  Demos.RegisterDemo('Dump', 'Using TPgDump component',
    'Demonstrates how to backup data from tables with the TPgDump component. ' +
    'Shows how to use scripts created during back up to restore table data. ' +
    'This demo lets you back up a table either by specifying the table name ' +
    'or by writing a SELECT query.',
    'Working with components', TDumpFrame, 4);
  Demos.RegisterDemo('Loader', 'Using TPgLoader component',
    'Uses the TPgLoader component to quickly load data into a server table. ' +
    'This demo also compares the two TPgLoader data loading handlers: GetColumnData and PutData.',
    'Working with components', TLoaderFrame, 5);
{$ENDIF}
  Demos.RegisterDemo('Query', 'Using TPgQuery component',
    'Demonstrates working with TPgQuery, which is one of the most useful PgDAC ' +
    'components. Includes many TPgQuery usage scenarios. Demonstrates how to ' +
    'execute queries in both standard and NonBlocking mode and how to edit data ' +
    'and export it to XML files. Note: This is a very good introductory demo. ' +
    'We recommend starting here when first becoming familiar with PgDAC.',
    'Working with components', TQueryFrame, 6);
  Demos.RegisterDemo('Sql', 'Using TPgSQL component',
    'Uses the TPgSQL component to execute SQL statements. Demonstrates how to ' +
    'work in a separate thread, in standard mode, in NonBlocking mode, and how ' +
    'to break long-duration query execution.',
    'Working with components', TSqlFrame, 7);
  Demos.RegisterDemo('StoredProc', 'Using TPgStoredProc component',
    'Uses the TPgStoredProc component to access an editable recordset from an ' +
    'PgSQL server stored procedure in the client application.',
    'Working with components', TStoredProcFrame, 8);
  Demos.RegisterDemo('Table', 'Using TPgTable component',
    'Demonstrates how to use TPgTable to work with data from a single table on ' +
    'the server without manually writing any SQL queries. Performs server-side ' +
    'data sorting and filtering and retrieves results for browsing and editing.',
    'Working with components', TTableFrame, 9);
  Demos.RegisterDemo('UpdateSQL', 'Using TPgUpdateSQL component',
    'Demonstrates using the TPgUpdateSQL component to customize update commands. ' +
    'Lets you optionally use TPgSQL and TPgQuery objects for carrying out ' +
    'insert, delete, query, and update commands.',
    'Working with components', TUpdateSQLFrame, 10);
  Demos.RegisterDemo('VirtualTable', 'Using TVirtualTable component',
    'Demonstrates working with the TVirtualTable component. This sample shows ' +
    'how to fill virtual dataset with data from other datasets, filter data by ' +
    'a given criteria, locate specified records, perform file operations, and ' +
    'change data and table structure.',
    'Working with components', TVirtualTableFrame, 11, 'VTable');

  Demos.RegisterDemo('CachedUpdates', 'Cached updates, transaction control',
    'Demonstrates how to perform the most important tasks of working with data ' +
    'in CachedUpdates mode, including highlighting uncommitted changes, managing ' +
    'transactions, and committing changes in a batch.',
    'General demos', TCachedUpdatesFrame, 1);
  Demos.RegisterDemo('FilterAndIndex', 'Using Filter and IndexFieldNames',
    'Demonstrates PgDAC''s local storage functionality. This sample shows how ' +
    'to perform local filtering, sorting and locating by multiple fields, ' +
    'including by calculated and lookup fields.',
    'General demos', TFilterAndIndexFrame, 1);
{$IFNDEF FPC}
  Demos.RegisterDemo('AggregateAndCalc', 'Using Calculated, InternalCalc and Aggregate Fields',
    'Shows how to work with Calculated, Internal Calculated and Aggregate fields.',
    'General demos', TAggregateFrame, 1);
{$ENDIF}
  Demos.RegisterDemo('MasterDetail', 'Master/detail relationship',
    'Uses PgDAC functionality to work with master/detail relationships. ' +
    'This sample shows how to use local master/detail functionality. ' +
    'Demonstrates different kinds of master/detail linking, including linking ' +
    'by SQL, by simple fields, and by calculated fields.',
    'General demos', TMasterDetailFrame, 1);
  Demos.RegisterDemo('Lock', 'Editing data with row-level locking',
    'Demonstrates two kinds of row-level locking (Update locking and Exclusive ' +
    'locking). This functionality is based on locking hints.',
    'General demos', TLockFrame, 1);
{$IFNDEF FPC}
  Demos.RegisterDemo('Pictures', 'Working with BLOB field',
    'Uses PgDAC functionality to work with graphics. The sample demonstrates ' +
    'how to retrieve binary data from PgSQL server database and display it on ' +
    'visual components. Sample also shows how to load and save pictures ' +
    'to files and to the database.',
    'PgSQL-specific', TPicturesFrame, 1);
{$ENDIF}
  Demos.RegisterDemo('Text', 'Working with long string fields', 'Uses PgDAC ' +
    'functionality to work with text. The sample demonstrates how to retrieve ' +
    'text data from SQL Server database and display it on visual components. ' +
    'Sample also shows how to load and save text to files and to the database.',
    'PgSQL-specific', TTextFrame, 1);

  Demos.RegisterCategory('Miscellaneous', '', -1, True);
{$IFDEF CLR}
  Demos.RegisterDemo('AspNet', 'AspNet', 'Uses PgDataAdapter to create a simple ' +
    'ASP .NET application. This demo shows how to create an ASP.NET application ' +
    'that lets you connect to a database and execute queries. Application ' +
    'displays query results in a DataGrid and sends user changes back to the database.',
    'Miscellaneous', nil, 1, '', True);
{$ENDIF}
{$IFNDEF CLR}
  Demos.RegisterDemo('Dll', 'Dll',
    'Demonstrates creating and loading DLLs for PgDAC-based projects. This demo ' +
    'project consists of two parts - an Pg_Dll project that creates a DLL of a ' +
    'form that sends a query to the server and displays its results, and an ' +
    'Pg_Exe project that can be executed to display a form for loading and ' +
    'running this DLL. Allows you to build a dll for one PgDAC-based project ' +
    'and load and test it from a separate application.',
    'Miscellaneous', nil, 1, '', True);
{$ENDIF}
  Demos.RegisterDemo('FailOver', 'FailOver',
    'Demonstrates the recommended approach to working with unstable networks. ' +
    'This sample lets you perform transactions and updates in several different modes, ' +
    'simulate a sudden session termination, and view what happens to your data ' +
    'state when connections to the server are unexpectedly lost. Shows off ' +
    'CachedUpdates, LocalMasterDetail, FetchAll, Pooling, and different Failover modes.',
    'Miscellaneous', nil, 1, '', True);
{$IFDEF CLR}
  Demos.RegisterDemo('WinForms', 'AspNet', 'Shows how to use PgDAC to create a ' +
    'WinForm application. This demo project creates a simple WinForms application ' +
    'and fills a data grid from an PgDataAdapter data source.',
    'Miscellaneous', nil, 1, '', True);
{$ENDIF}

{$IFNDEF CLR}
  Demos.RegisterCategory('', '', -1, True);
{$ENDIF}
  Demos.RegisterCategory('TechnologySpecific', '', -1, True);
{$IFNDEF CLR}
  Demos.RegisterCategory('', '', -1, True);
{$ENDIF}

{$IFNDEF CLR}
  Demos.RegisterCategory('ThirdParty', '', -1, True);
  Demos.RegisterDemo('FastReport', 'FastReport hint',
    'Demonstrates how PgDAC can be used with FastReport components. ' +
    'This project consists of two parts. The first part is several packages that ' +
    'integrate PgDAC components into the FastReport editor. The second part is ' +
    'a demo application that lets you design and preview reports with PgDAC ' +
    'technology in the FastReport editor.',
    'ThirdParty', nil, 1, '', True);
{$ENDIF}
{$IFNDEF CLR}
  Demos.RegisterCategory('', '', -1, True);
{$ENDIF}

end;

procedure TPgDacForm.lbAboutClick(Sender: TObject);
begin
  inherited;
  PgDacAboutForm.ShowModal;
end;

procedure TPgDacForm.cbDebugClick(Sender: TObject);
begin
  inherited;
  scCreate.Debug := (Sender as TCheckBox).Checked;
  scDrop.Debug := (Sender as TCheckBox).Checked;
end;

{$IFDEF FPC}
initialization
  {$i PgDacDemoForm.lrs}
{$ENDIF}

end.

