unit Table;

interface

uses
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBCtrls, ExtCtrls, Db, Grids, DBGrids, StdCtrls, ToolWin,
  ComCtrls, Buttons, PgDACVcl, DemoFrame, PgDacDemoForm,
  CRTypes, CRFunctions, {$IFNDEF FPC}MemDS,{$ELSE}MemDataSet,{$ENDIF}
  DBAccess, PgAccess;

type
  TTableFrame = class(TDemoFrame)
    DataSource: TDataSource;
    PgTable: TPgTable;
    DBGrid: TDBGrid;
    ToolBar: TPanel;
    Panel3: TPanel;
    btOpen: TSpeedButton;
    btClose: TSpeedButton;
    btPrepare: TSpeedButton;
    btUnPrepare: TSpeedButton;
    btExecute: TSpeedButton;
    DBNavigator: TDBNavigator;
    Panel7: TPanel;
    Label3: TLabel;
    edOrderFields: TEdit;
    Panel5: TPanel;
    Label1: TLabel;
    edTableName: TComboBox;
    Panel8: TPanel;
    Label4: TLabel;
    edFilterSQL: TEdit;
    procedure btOpenClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btPrepareClick(Sender: TObject);
    procedure btUnPrepareClick(Sender: TObject);
    procedure btExecuteClick(Sender: TObject);
    procedure edTableNameExit(Sender: TObject);
    procedure edTableNameDropDown(Sender: TObject);
  private
    { Private declarations }
    procedure ShowState;
    procedure AssignProperties;
  public
    // Demo management
    procedure Initialize; override;
    procedure SetDebug(Value: boolean); override;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

procedure TTableFrame.AssignProperties;
begin
  try
    if Trim(PgTable.TableName) <> Trim(edTableName.Text) then
      PgTable.TableName := edTableName.Text;

    PgTable.FilterSQL := edFilterSQL.Text;

    try
      PgTable.OrderFields:= edOrderFields.Text;
    except
      edOrderFields.SetFocus;
      raise;
    end;
  finally
    ShowState;
  end;

end;

procedure TTableFrame.ShowState;
var
  St: string;

  procedure AddSt(S: string);
  begin
    if St <> '' then
      St:= St + ', ';
    St:= St + S;
  end;

begin
  St:= '';

  if PgTable.Prepared then begin
    AddSt('Prepared');

    if PgTable.IsQuery then
      AddSt('IsQuery');
  end;

  if PgTable.Active then
    AddSt('Active')
  else
    AddSt('Inactive');

  PgDacForm.StatusBar.Panels[2].Text:= St;
end;

procedure TTableFrame.btOpenClick(Sender: TObject);
begin
  try
    AssignProperties;
    PgTable.Open;
  finally
    ShowState;
  end;
end;

procedure TTableFrame.btCloseClick(Sender: TObject);
begin
  PgTable.Close;
  ShowState;
end;

procedure TTableFrame.btPrepareClick(Sender: TObject);
begin
  try
    AssignProperties;
    PgTable.Prepare;
  finally
    ShowState;
  end;
end;

procedure TTableFrame.btUnPrepareClick(Sender: TObject);
begin
  PgTable.UnPrepare;
  ShowState;
end;

procedure TTableFrame.btExecuteClick(Sender: TObject);
begin
  try
    AssignProperties;
    PgTable.Execute;
  finally
    ShowState;
  end;
end;

procedure TTableFrame.edTableNameExit(Sender: TObject);
begin
  AssignProperties;
end;

procedure TTableFrame.edTableNameDropDown(Sender: TObject);
var
  List: TStringList;
begin
  List := TStringList.Create;
  try
    PgTable.Connection.GetTableNames(List);
    AssignStrings(List, edTableName.Items);
  finally
    List.Free;
  end;
end;

// Demo management
procedure TTableFrame.Initialize;
begin
  inherited;
  PgTable.Connection := Connection as TPgConnection;
  edTableName.Text:= PgTable.TableName;
  edOrderFields.Text:= PgTable.OrderFields;
  ShowState;
end;

procedure TTableFrame.SetDebug(Value: boolean);
begin
  PgTable.Debug:= Value;
end;

{$IFDEF FPC}
initialization
  {$i Table.lrs}
{$ENDIF}
end.
