
//////////////////////////////////////////////////
//  PostgreSQL Data Access Components
//  Copyright  2008-2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}
{$I PgDac.inc}

unit PgDesign;
{$ENDIF}
interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, TypInfo, DB,
{$IFDEF CLR}
  Borland.Vcl.Design.DesignEditors, Borland.Vcl.Design.DesignIntf,
  Borland.Vcl.Design.FldLinks, WinUtils,
{$ELSE}
{$IFDEF FPC}
  PropEdits, ComponentEditors,
{$ELSE}
  DesignIntf, DesignEditors,
  {$IFNDEF BCB}FldLinks, ColnEdit,{$ENDIF}
{$ENDIF}
{$ENDIF}
  CRFunctions, CRTypes, DBAccess, PgAccess, PgScript,
{$IFNDEF STD}
  PgLoader, PgDump, PgAlerter,
{$ENDIF}
  PgDacVcl, CRDesign, DADesign;

procedure Register;

type
{$IFNDEF FPC}
  TPgConnectionList = class (TDAConnectionList)
  protected
    function GetConnectionType: TCustomDAConnectionClass; override;
  end;

  TPgDesignNotification = class(TDADesignNotification)
  public
    procedure ItemInserted(const ADesigner: IDesigner; AItem: TPersistent); override;
    function CreateConnectionList: TDAConnectionList; override;
    function GetConnectionPropertyName: string; override;
  end;
{$ENDIF}

  TPgTableNameEditor = class (TDATableNameEditor)
  protected
    function InternalGetValues(Proc: TGetStrProc): TCustomDAConnection; override;
  end;

  TPgSPNameEditor = class (TDASPNameEditor)
  protected
    function InternalGetValues(Proc: TGetStrProc): TCustomDAConnection; override;
  end;

  TPgConnectDialogPropertyEditor = class(TComponentProperty)
  private
    FCheckProc: TGetStrProc;
    procedure CheckComponent(const Value: string);
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TPgConnectionEditor = class (TDAConnectionEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgQueryEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgTableEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgStoredProcEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgSQLEditor = class (TDASQLEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgScriptEditor = class (TDAScriptEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgUpdateSQLEditor = class (TDAUpdateSQLEditor)
  protected
    procedure InitVerbs; override;
  end;

  TPgCharsetEditor = class(TStringProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

implementation

uses
{$IFNDEF FPC}
  PgDacMenu,
{$ENDIF}
  PgDesignUtils, PgConnectionEditor, PgQueryEditor, PgSQLEditor,
  PgTableEditor, PgStoredProcEditor, DAScriptEditor, PgUpdateSQLEditor,
  PgTransaction;

{$IFDEF FPC}
type
  TBlobTypes = ftBlob..ftWideMemo;
{$ENDIF}

var
  CharsetList: TStringList;
{$IFNDEF FPC}
  Notificator: IDesignNotification;
{$ENDIF}

procedure Register;
begin
// Register property editors
  RegisterPropertyEditor(TypeInfo({$IFNDEF FPC}TBlobType{$ELSE}TBlobTypes{$ENDIF}), TPgLargeObjectField, 'BlobType', nil);

  RegisterPropertyEditor(TypeInfo(TCustomConnectDialog), TPgConnection, 'ConnectDialog', TPgConnectDialogPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQLDelete', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQLInsert', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQLLock', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQLRecCount', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQLRefresh', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgQuery, 'SQLUpdate', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TDAParams), TPgTable, 'Params', nil);
  RegisterPropertyEditor(TypeInfo(string),    TPgTable, 'TableName', TPgTableNameEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TPgSQL, 'SQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TPgScript, 'SQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQL', nil);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQLDelete', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQLInsert', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQLRecCount', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQLRefresh', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQLUpdate', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgStoredProc, 'SQLLock', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(string),   TPgStoredProc, 'StoredProcName', TPgSPNameEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TPgUpdateSQL, 'InsertSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgUpdateSQL, 'ModifySQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgUpdateSQL, 'DeleteSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPgUpdateSQL, 'RefreshSQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(String), TPgConnectionOptions, 'Charset', TPgCharsetEditor);
  RegisterPropertyEditor(TypeInfo(String), TPgConnectionOptions, 'MessagesCharset', TPgCharsetEditor);

// Register component editors
  DARegisterComponentEditor(TPgConnection, TPgConnectionEditor, TPgConnectionEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgQuery, TPgQueryEditor, TPgQueryEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgSQL, TPgSQLEditor, TPgSQLEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgTable, TPgTableEditor, TPgTableEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgStoredProc, TPgStoredProcEditor, TPgStoredProcEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgScript, TPgScriptEditor, TDAScriptEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgMetaData, TDAComponentEditor, nil, TPgDesignUtils);
  DARegisterComponentEditor(TPgUpdateSQL, TPgUpdateSQLEditor, TPgUpdateSQLEditorForm, TPgDesignUtils);
  DARegisterComponentEditor(TPgTransaction, TDAComponentEditor, nil, TPgDesignUtils);
{$IFNDEF FPC}
  DARegisterComponentEditor(TPgDataSource, TCRDataSourceEditor, nil, TPgDesignUtils);
{$ENDIF}
{$IFNDEF STD}
  RegisterPropertyEditor(TypeInfo(string), TPgLoader, 'TableName', TDALoaderTableNameEditor);
  DARegisterComponentEditor(TPgLoader, TDALoaderEditor, nil, TPgDesignUtils);
  DARegisterComponentEditor(TPgAlerter, TDAComponentEditor, nil, TPgDesignUtils);
  DARegisterComponentEditor(TPgDump, TDAComponentEditor, nil, TPgDesignUtils);
{$ENDIF}

{$IFDEF MSWINDOWS}
{$IFNDEF FPC}
  Menu.AddItems({$IFDEF CLR}WinUtils{$ELSE}SysInit{$ENDIF}.HInstance);
{$ENDIF}
{$ENDIF}
end;

{$IFNDEF FPC}

{ TPgConnectionList }

function TPgConnectionList.GetConnectionType: TCustomDAConnectionClass;
begin
  Result := TPgConnection;
end;

{ TPgDesignNotification }

procedure TPgDesignNotification.ItemInserted(const ADesigner: IDesigner; AItem: TPersistent);
begin
  if (AItem <> nil) and ((AItem is TCustomPgDataSet)
    or (AItem is TPgScript)
    or (AItem is TPgDataSource)
    or (AItem is TPgSQL)
    or (AItem is TPgMetaData)
  {$IFNDEF STD}
    or (AItem is TPgLoader)
    or (AItem is TPgDump)
    or (AItem is TPgAlerter)
    or (AItem is TPgTransaction)
  {$ENDIF}
    )
  then
    FItem := AItem;
end;

function TPgDesignNotification.CreateConnectionList: TDAConnectionList;
begin
  Result := TPgConnectionList.Create;
end;

function TPgDesignNotification.GetConnectionPropertyName: string;
begin
{$IFNDEF STD}
  if FItem is TPgTransaction then
    Result := 'DefaultConnection'
  else
{$ENDIF}
  Result := 'Connection';
end;
{$ENDIF}

{ TPgTableNameEditor }

function TPgTableNameEditor.InternalGetValues(Proc: TGetStrProc): TCustomDAConnection;
var
  List: TStringList;
  i: integer;
  Component: TComponent;
begin
  Assert(PropCount > 0, 'PropCount = 0');
  Component := GetComponent(0) as TComponent;
  Assert(Component is TCustomDADataSet, Component.ClassName);

  Result := TDBAccessUtils.UsedConnection(TCustomDADataSet(Component));
  if Result = nil then
    Exit;

  List := TStringList.Create;
  try
    (Result as TPgConnection).GetTableNames(List, False, False, False, True);
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TPgSPNameEditor }

function TPgSPNameEditor.InternalGetValues(Proc: TGetStrProc): TCustomDAConnection;
var
  List: TStringList;
  i: integer;
  Component: TComponent;
begin
  Assert(PropCount > 0, 'PropCount = 0');
  Component := GetComponent(0) as TComponent;
  Assert(Component is TCustomDADataSet, Component.ClassName);

  Result := TDBAccessUtils.UsedConnection(TCustomDADataSet(Component));
  if Result = nil then
    Exit;

  List := TStringList.Create;
  try
    (Result as TPgConnection).GetStoredProcNames(List, False, False, True);
    List.Sort;
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TPgCharsetEditor }

procedure TPgCharsetEditor.GetValues(Proc: TGetStrProc);
var
  i: integer;
begin
  for i := 0 to CharsetList.Count - 1 do
    Proc(CharsetList.Strings[i]);
end;

function TPgCharsetEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

{ TPgConnectDialogPropertyEditor }

procedure TPgConnectDialogPropertyEditor.CheckComponent(const Value: string);
var
  Component: TComponent;
begin
  Component := {$IFDEF FPC}PropertyHook{$ELSE}Designer{$ENDIF}.GetComponent(Value);
  if Component <> nil then begin
    if not ({$IFDEF VER17P}(GetIsClassByName(Component, 'TPgConnectDialogFmx')) or {$ENDIF}(Component is TPgConnectDialog)) then
      Exit;
  end;
  FCheckProc(Value);
end;

procedure TPgConnectDialogPropertyEditor.GetValues(Proc: TGetStrProc);
begin
  FCheckProc := Proc;
  inherited GetValues(CheckComponent);
end;

{ Component editors }

procedure TPgConnectionEditor.InitVerbs;
begin
  inherited;

  AddVerb('Connection Editor...', TPgConnectionEditorForm, TPgDesignUtils);
end;

procedure TPgQueryEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('PgQuery E&ditor...', TPgQueryEditorForm, TPgDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);
  inherited;
end;

procedure TPgTableEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('PgTable E&ditor...', TPgTableEditorForm, TPgDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);

  inherited;
end;

procedure TPgStoredProcEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('StoredProc E&ditor...', TPgStoredProcEditorForm, TPgDesignUtils);

  inherited;
end;

procedure TPgSQLEditor.InitVerbs;
begin
  inherited;

  AddVerb('PgSQL E&ditor...', TPgSQLEditorForm, TPgDesignUtils);
end;

procedure TPgScriptEditor.InitVerbs;
begin
  inherited;

  AddVerb('PgScript E&ditor...', TDAScriptEditorForm, TPgDesignUtils);
end;

procedure TPgUpdateSQLEditor.InitVerbs;
begin
  inherited;
  AddVerb('PgUpdateSQL E&ditor...', TPgUpdateSQLEditorForm, TPgDesignUtils);
end;

initialization
{$IFNDEF FPC}
  RegisterClass(TPgTimeStampField);
  RegisterClass(TPgDateField);
  RegisterClass(TPgTimeField);
  RegisterClass(TPgIntervalField);
  RegisterClass(TPgLargeObjectField);
  RegisterClass(TPgGeometricField);
  RegisterClass(TPgCursorField);
  RegisterClass(TPgGisField);
  Notificator := TPgDesignNotification.Create;
  RegisterDesignNotification(Notificator);
{$ENDIF}

  CharsetList := TStringList.Create;
  CharsetList.Add('BIG5');
  CharsetList.Add('EUC_CN');
  CharsetList.Add('EUC_JP');
  CharsetList.Add('EUC_JIS_2004');
  CharsetList.Add('EUC_KR');
  CharsetList.Add('EUC_TW');
  CharsetList.Add('GB18030');
  CharsetList.Add('GBK');
  CharsetList.Add('ISO_8859_5');
  CharsetList.Add('ISO_8859_6');
  CharsetList.Add('ISO_8859_7');
  CharsetList.Add('ISO_8859_8');
  CharsetList.Add('JOHAB');
  CharsetList.Add('KOI8R');
  CharsetList.Add('KOI8U');
  CharsetList.Add('LATIN1');
  CharsetList.Add('LATIN2');
  CharsetList.Add('LATIN3');
  CharsetList.Add('LATIN4');
  CharsetList.Add('LATIN5');
  CharsetList.Add('LATIN6');
  CharsetList.Add('LATIN7');
  CharsetList.Add('LATIN8');
  CharsetList.Add('LATIN9');
  CharsetList.Add('LATIN10');
  CharsetList.Add('MULE_INTERNAL');
  CharsetList.Add('SJIS');
  CharsetList.Add('SHIFT_JIS_2004');
  CharsetList.Add('SQL_ASCII');
  CharsetList.Add('UHC');
  CharsetList.Add('UTF8');
  CharsetList.Add('WIN866');
  CharsetList.Add('WIN874');
  CharsetList.Add('WIN1250');
  CharsetList.Add('WIN1251');
  CharsetList.Add('WIN1252');
  CharsetList.Add('WIN1253');
  CharsetList.Add('WIN1254');
  CharsetList.Add('WIN1255');
  CharsetList.Add('WIN1256');
  CharsetList.Add('WIN1257');
  CharsetList.Add('WIN1258');
    
finalization
{$IFNDEF FPC}
  UnRegisterDesignNotification(Notificator);
  Notificator := nil;
{$ENDIF}

  CharsetList.Free;

end.
