
//////////////////////////////////////////////////
//  PostgreSQL Data Access Components
//  Copyright  2008-2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I PgDac.inc}

unit PgDesignUtils;
{$ENDIF}

interface

uses
  Classes, SysUtils, CRTypes, CRDataTypeMap, CRDesignUtils, DBAccess, DADesignUtils;

type
  TPgDesignUtils = class(TDADesignUtils)
    class function GetProjectName: string; override;

  { Component }
  {$IFDEF PGDAC}
    class function GetConnectionList: TObject; override;
  {$ENDIF}
    class function GetDesignCreate(Obj: TComponent): boolean; override;
    class procedure SetDesignCreate(Obj: TComponent; Value: boolean); override;

  { Connection support }
    class function HasConnection(Obj: TComponent): boolean; override;
    class function GetConnection(Obj: TComponent): TCustomDAConnection; override;
    class procedure SetConnection(Obj: TComponent; Value: TCustomDAConnection); override;

  { TDATable support }
    class function GetTableName(Obj: TCustomDADAtaSet): string; override;
    class procedure SetTableName(Obj: TCustomDADAtaSet; const Value: string); override;
    class function GetOrderFields(Obj: TCustomDADAtaSet): string; override;
    class procedure SetOrderFields(Obj: TCustomDADAtaSet; const Value: string); override;
    class procedure PrepareSQL(Obj: TCustomDADAtaSet); override;
    class function GetStoredProcName(Obj: TCustomDADataSet): string; override;
    class procedure SetStoredProcName(Obj: TCustomDADataSet; const Value: string); override;

  { Data Type Mapping }
    class function GetConverterManagerClass: TConverterManagerClass; override;
  end;

  TPgDesignUtilsClass = class of TPgDesignUtils;

implementation

uses
{$IFDEF PGDAC}
  PgDesign,
{$ENDIF}
{$IFNDEF STD}
  DALoader, PgLoader, PgDump, DADump, PgAlerter,
{$ENDIF}
  PgAccess, PgDataTypeMap;

{ TPgDesignUtils }

class function TPgDesignUtils.GetProjectName: string;
begin
  Result := 'PgDAC';
end;

class function TPgDesignUtils.HasConnection(Obj: TComponent): boolean;
begin
  Result := inherited HasConnection(Obj);
end;

class function TPgDesignUtils.GetConnection(Obj: TComponent): TCustomDAConnection;
begin
  Result := inherited GetConnection(Obj);
end;

class procedure TPgDesignUtils.SetConnection(Obj: TComponent; Value: TCustomDAConnection);
begin
  inherited;
end;

class function TPgDesignUtils.GetOrderFields(Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TCustomPgTable, Obj.ClassName);
  Result := TCustomPgTable(Obj).OrderFields;
end;

class procedure TPgDesignUtils.SetOrderFields(Obj: TCustomDADAtaSet;
  const Value: string);
begin
  Assert(Obj is TCustomPgTable, Obj.ClassName);
  TCustomPgTable(Obj).OrderFields := Value;
end;

class function TPgDesignUtils.GetTableName(Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TCustomPgTable, Obj.ClassName);
  Result := TCustomPgTable(Obj).TableName;
end;

class procedure TPgDesignUtils.SetTableName(Obj: TCustomDADAtaSet; const Value: string);
begin
  Assert(Obj is TCustomPgTable, Obj.ClassName);
  TCustomPgTable(Obj).TableName := Value;
end;

class procedure TPgDesignUtils.PrepareSQL(Obj: TCustomDADAtaSet);
begin
  Assert(Obj is TCustomPgTable, Obj.ClassName);
  TCustomPgTable(Obj).PrepareSQL;
end;

class function TPgDesignUtils.GetStoredProcName(Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TCustomPgStoredProc, Obj.ClassName);
  Result := TCustomPgStoredProc(Obj).StoredProcName + ':' +
    IntToStr(TCustomPgStoredProc(Obj).Overload);
end;

class procedure TPgDesignUtils.SetStoredProcName(Obj: TCustomDADAtaSet;
  const Value: string);
var
  p: integer;
begin
  Assert(Obj is TCustomPgStoredProc, Obj.ClassName);

  p := Pos(':', Value);
  if p > 0 then begin
    TCustomPgStoredProc(Obj).StoredProcName := Copy(Value, 1, p - 1);
    TCustomPgStoredProc(Obj).Overload := StrToInt(Copy(Value, p + 1, MaxInt));
  end
  else
    TCustomPgStoredProc(Obj).StoredProcName := Value;
end;

class function TPgDesignUtils.GetConverterManagerClass: TConverterManagerClass;
begin
  Result := TConverterManagerClass(TPgMapRules.GetConverterManager.ClassType);
end;

{$IFDEF PGDAC}
class function TPgDesignUtils.GetConnectionList: TObject;
begin
  Result := TPgConnectionList.Create;
end;
{$ENDIF}

class function TPgDesignUtils.GetDesignCreate(Obj: TComponent): boolean;
begin
  Result := inherited GetDesignCreate(Obj);
end;

class procedure TPgDesignUtils.SetDesignCreate(Obj: TComponent; Value: boolean);
begin
  inherited;
end;

end.
