
//////////////////////////////////////////////////
//  PostgreSQL Data Access Components
//  Copyright  2008-2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}

unit PgConnectForm;
{$ENDIF}
interface

uses
  Classes, SysUtils,
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, StdCtrls, ExtCtrls, Forms, Dialogs,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  CRTypes, MemUtils, DBAccess, PgAccess, TypInfo;

type
  TPgConnectForm = class(TForm)
    Panel: TPanel;
    pUserName: TPanel;
    lbUsername: TLabel;
    edUsername: TEdit;
    pPassword: TPanel;
    pServer: TPanel;
    pPort: TPanel;
    pDatabase: TPanel;
    edPassword: TEdit;
    lbPassword: TLabel;
    lbServer: TLabel;
    edServer: TComboBox;
    lbPort: TLabel;
    edPort: TEdit;
    lbDatabase: TLabel;
    edDatabase: TComboBox;
    pButtons: TPanel;
    btCancel: TButton;
    btConnect: TButton;
    pSchema: TPanel;
    lbSchema: TLabel;
    edSchema: TEdit;
    procedure btConnectClick(Sender: TObject);
    procedure edServerDropDown(Sender: TObject);
    procedure edDatabaseDropDown(Sender: TObject);
    procedure edExit(Sender: TObject);

  private
    FConnectDialog: TCustomConnectDialog;
    FRetries: integer;
    FRetry: boolean;

    FListGot: boolean;

    procedure SetConnectDialog(Value: TCustomConnectDialog);

  protected
    procedure DoInit; virtual;
    procedure DoConnect; virtual;

  public

  published
    property ConnectDialog: TCustomConnectDialog read FConnectDialog write SetConnectDialog;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF IDE}
{$R *.dfm}
{$ELSE}
{$R PgConnectForm.dfm}
{$ENDIF}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  CRFunctions, DacVcl, PgDacVcl;

procedure TPgConnectForm.DoInit;
var
  CurTop, i: integer;
  ConnectDialogOptions: TConnectDialogOptionArray;
  CurPanel: TPanel;
  CurControl: TWinControl;
  NeedSetActiveControl: Boolean;
begin
  FRetry := False;
  FRetries := FConnectDialog.Retries;
  Caption := FConnectDialog.Caption;
  FListGot := False;

  FConnectDialog.GetOptions(ConnectDialogOptions);
  CurTop := 0;
  NeedSetActiveControl := True;
  for i := Low(ConnectDialogOptions) to High(ConnectDialogOptions) do begin
    CurPanel := nil;
    CurControl := nil;
    case ConnectDialogOptions[i].Kind of
      okServer: begin
        pServer.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pServer;
        CurControl := edServer;
        lbServer.Caption := ConnectDialogOptions[i].Caption;
      end;
      okUserName: begin
        pUserName.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pUserName;
        CurControl := edUsername;
        lbUsername.Caption := ConnectDialogOptions[i].Caption;
      end;
      okPassword: begin
        pPassword.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pPassword;
        CurControl := edPassword;
        lbPassword.Caption := ConnectDialogOptions[i].Caption;
      end;
      okDatabase: begin
        pDatabase.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pDatabase;
        CurControl := edDatabase;
        lbDatabase.Caption := ConnectDialogOptions[i].Caption;
      end;
      okPort: begin
        pPort.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pPort;
        CurControl := edPort;
        lbPort.Caption := ConnectDialogOptions[i].Caption;
      end;
      okSchema: begin
        pSchema.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pSchema;
        CurControl := edSchema;
        lbSchema.Caption := ConnectDialogOptions[i].Caption;
      end;
    else
      Assert(False);
    end;
    CurPanel.TabOrder := i;
    if CurPanel.Visible then begin
      if i = 0 then
        CurPanel.Top := CurTop + 2
      else
        CurPanel.Top := CurTop;
      CurTop := CurTop + CurPanel.Height;
    end;
    if NeedSetActiveControl and CurPanel.Visible then begin
      ActiveControl := CurControl;
      NeedSetActiveControl := False;
    end;
  end;
  Panel.Height := CurTop + 2;
  pButtons.Top := Panel.Height + 8;
  ClientHeight := pButtons.Top + pButtons.Height + 2;

  btConnect.Caption := FConnectDialog.ConnectButton;
  btCancel.Caption := FConnectDialog.CancelButton;

  edUsername.Text := FConnectDialog.Connection.Username;
  edPassword.Text := FConnectDialog.Connection.Password;
  edServer.Text := FConnectDialog.Connection.Server;

  if (edUsername.Text <> '') and (edPassword.Text = '') and pPassword.Visible then
    ActiveControl := edPassword;

  if FConnectDialog.Connection is TPgConnection then begin
    edDatabase.Text := TPgConnection(FConnectDialog.Connection).Database;
    edPort.Text := IntToStr(TPgConnection(FConnectDialog.Connection).Port);
    edSchema.Text := TPgConnection(FConnectDialog.Connection).Schema;
  end;
end;

procedure TPgConnectForm.DoConnect;
begin
  try
    edExit(nil);
    FConnectDialog.Connection.PerformConnect(FRetry);
    ModalResult := mrOk;
  except
    on E: EDAError do begin
      Dec(FRetries);
      FRetry := True;
      if FRetries = 0 then
        ModalResult:= mrCancel;

      {case E.ErrorCode of
        ER_ACCESS_DENIED_ERROR: ActiveControl := edUsername;
        CR_UNKNOWN_HOST: ActiveControl := edServer;
      end;}
      raise;
    end
    else
      raise;
  end;
end;

procedure TPgConnectForm.SetConnectDialog(Value: TCustomConnectDialog);
begin
  FConnectDialog := Value;
  DoInit;
end;

procedure TPgConnectForm.btConnectClick(Sender: TObject);
begin
  DoConnect;
end;

procedure TPgConnectForm.edServerDropDown(Sender: TObject);
var
  OldCursor: TCursor;
  List: TStringList;
begin
  if FListGot then
    Exit;

  FListGot := True;
  OldCursor := GetScreenCursor;
  SetScreenCursor(crSQLWait);
  List := TStringList.Create;
  try
    FConnectDialog.GetServerList(List);
    AssignStrings(List, edServer.Items);
  finally
    List.Free;
    SetScreenCursor(OldCursor);
  end;
end;

procedure TPgConnectForm.edDatabaseDropDown(Sender: TObject);
var
  Connection: TPgConnection;
  OldCursor: TCursor;
  OldLoginPrompt: boolean;
  OldConnected: boolean;
  OldDatabase: string;
  List: TStringList;
begin
  Connection := FConnectDialog.Connection as TPgConnection;
  OldLoginPrompt := Connection.LoginPrompt;
  OldCursor := GetScreenCursor;
  OldConnected := Connection.Connected;
  OldDatabase := Connection.Database;

  Connection.UserName := edUsername.Text;
  Connection.Password := edPassword.Text;
  Connection.Server := edServer.Text;
  Connection.Port := StrToInt(edPort.Text);
  if not OldConnected then
    Connection.Database := 'template1';

  Connection.LoginPrompt := False;
  SetScreenCursor(crSQLWait);

  List := TStringList.Create;
  try
    TDBAccessUtils.SetLockLoginPrompt(Connection, True);
    try
      Connection.GetDatabaseNames(List);
      AssignStrings(List, edDatabase.Items);
    finally
      TDBAccessUtils.SetLockLoginPrompt(Connection, False);
    end;
  finally
    Connection.Connected := OldConnected;
    Connection.LoginPrompt := OldLoginPrompt;
    if not OldConnected then
      Connection.Database := OldDatabase;
    Screen.Cursor := OldCursor;
    List.Free;
  end;
end;

procedure TPgConnectForm.edExit(Sender: TObject);
begin
  try
    if pPassword.Visible then
      FConnectDialog.Connection.Password := edPassword.Text;
    if pServer.Visible then
      FConnectDialog.Connection.Server := edServer.Text;
    if pUserName.Visible then
      FConnectDialog.Connection.UserName := edUsername.Text;

    if FConnectDialog.Connection is TPgConnection then begin
      if pDatabase.Visible then
        TPgConnection(FConnectDialog.Connection).Database := edDatabase.Text;
      if pPort.Visible then
        TPgConnection(FConnectDialog.Connection).Port := StrToInt(edPort.Text);
      if pSchema.Visible then
        TPgConnection(FConnectDialog.Connection).Schema := edSchema.Text;
    end;
  except
    ActiveControl := Sender as TWinControl;
    raise;
  end;
end;

initialization
  if GetClass('TPgConnectForm') = nil then
    Classes.RegisterClass(TPgConnectForm);

end.

