type

{ TPgConnectDialog }

{$IFDEF VER16P}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64{$IFNDEF STD} or pidOSX32{$IFDEF VER25P} or pidLinux64{$IFDEF VER26P} or pidOSX64{$ENDIF}{$ENDIF}{$ENDIF})]
{$ENDIF}
  {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF} = class(TCustomConnectDialog)
  private
    FDatabaseOption: TConnectDialogOption;
    FPortOption: TConnectDialogOption;
    FSchemaOption: TConnectDialogOption;

    function GetDatabaseLabel: string;
    procedure SetDatabaseLabel(Value: string);
    function GetPortLabel: string;
    procedure SetPortLabel(Value: string);
    function GetShowDatabase: Boolean;
    procedure SetShowDatabase(Value: Boolean);
    function GetShowPort: Boolean;
    procedure SetShowPort(Value: Boolean);

    procedure SetDatabaseOption(Value: TConnectDialogOption);
    procedure SetPortOption(Value: TConnectDialogOption);
    procedure SetSchemaOption(Value: TConnectDialogOption);

    function GetConnection: TPgConnection;

  protected
    procedure SetLabelSet(Value: TLabelSet); override;
    function GetServerEnumeratorClass: TCRServerEnumeratorClass; override;
    function DefDialogClass: TClass; override;
  {$IFDEF MSWINDOWS}
    function GetKeyPath: string; override;
    function GetServerStoreName: string; override;
    procedure SaveInfoToRegistry(Registry: TRegistry); override;
    procedure LoadInfoFromRegistry(Registry: TRegistry); override;
  {$ENDIF}

    procedure ReadDatabaseCaptionProperty(Reader: TReader);
    procedure ReadPortCaptionProperty(Reader: TReader);
    procedure ReadShowDatabaseProperty(Reader: TReader);
    procedure ReadShowPortProperty(Reader: TReader);
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: boolean; override;

    procedure GetServerList(List: TStrings); override;

    property Connection: TPgConnection read GetConnection;

    procedure GetOptions(var Options: TConnectDialogOptionArray; Ordered: Boolean = True); override;

    property DatabaseLabel: string read GetDatabaseLabel write SetDatabaseLabel;
    property PortLabel: string read GetPortLabel write SetPortLabel;
    property ShowDatabase: boolean read GetShowDatabase write SetShowDatabase default True;
    property ShowPort: boolean read GetShowPort write SetShowPort default True;
  published
    property Retries;
    property SavePassword;
    property DialogClass;

    property Caption;
    property ConnectButton;
    property CancelButton;

    property Server;
    property UserName;
    property Password;
    property Database: TConnectDialogOption read FDatabaseOption write SetDatabaseOption;
    property Port: TConnectDialogOption read FPortOption write SetPortOption;
    property Schema: TConnectDialogOption read FSchemaOption write SetSchemaOption;

    property LabelSet;
    property StoreLogInfo;
  end;

function DefConnectDialogClass: TClass;

implementation

uses
  {$IFDEF VER16P}{$IFDEF FMX}FMX.Types,{$ELSE}Controls,{$ENDIF}{$ENDIF}
  DAConsts, PgServerEnumerator;

function DefConnectDialogClass: TClass;
begin
  Result := {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF};
end;

const
  KeyPath = '\SOFTWARE\Devart\PgDAC\';

{ TPgConnectDialog }

constructor {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.Create(Owner: TComponent);
begin
  inherited;

  Server.Order := 0;
  UserName.Order := 2;
  Password.Order := 3;
  FDatabaseOption := TConnectDialogOption.Create(Self, okDatabase, 4);
  FPortOption := TConnectDialogOption.Create(Self, okPort, 1);
  FSchemaOption := TConnectDialogOption.Create(Self, okSchema, 5, False);
  LabelSet := lsEnglish;
end;

destructor {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.Destroy;
begin
  FDatabaseOption.Free;
  FPortOption.Free;
  FSchemaOption.Free;

  inherited;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.ReadDatabaseCaptionProperty(Reader: TReader);
begin
  FDatabaseOption.Caption := Reader.ReadString;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.ReadPortCaptionProperty(Reader: TReader);
begin
  FPortOption.Caption := Reader.ReadString;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.ReadShowDatabaseProperty(Reader: TReader);
begin
  FDatabaseOption.Visible := Reader.ReadBoolean;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.ReadShowPortProperty(Reader: TReader);
begin
  FPortOption.Visible := Reader.ReadBoolean;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.DefineProperties(Filer: TFiler);
begin
  inherited;

  Filer.DefineProperty('DatabaseLabel', ReadDatabaseCaptionProperty, nil, False);
  Filer.DefineProperty('PortLabel', ReadPortCaptionProperty, nil, False);
  Filer.DefineProperty('ShowDatabase', ReadShowDatabaseProperty, nil, False);
  Filer.DefineProperty('ShowPort', ReadShowPortProperty, nil, False);
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.Execute: boolean;
var
  OldDatabase, OldSchema: string;
  OldPort: integer;
begin
  if Connection = nil then
    DatabaseError(SConnectionNotDefined);
  Assert(Connection is TPgConnection);

  Result := False;
  OldDatabase := TPgConnection(Connection).Database;
  OldPort := TPgConnection(Connection).Port;
  OldSchema := TPgConnection(Connection).Schema;
  try
    Result := inherited Execute;
  finally
    if not Result then begin
      TPgConnection(Connection).Database := OldDatabase;
      TPgConnection(Connection).Port := OldPort;
      TPgConnection(Connection).Schema := OldSchema;
    end;
  end;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetServerList(List: TStrings);
begin
{$IFDEF MSWINDOWS}
  if UseServerHistory then
    LoadServerListFromRegistry(List)
  else begin
    CheckServerEnumerator;
    SetServerEnumerator(FServerEnumerator);
    FServerEnumerator.GetServerList(List);
    if List.Count = 0 then
      LoadServerListFromRegistry(List);
  end;
{$ENDIF}
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetLabelSet(Value: TLabelSet);
begin
  inherited SetLabelSet(Value);

  InSetLabelSet := True;
  try
  {$IFDEF WIN32_64}
    if Value <> lsCustom then begin
      FDatabaseOption.Caption := GetString(6);
      FPortOption.Caption := GetString(7);
    end;
  {$ELSE}
      FDatabaseOption.Caption := 'Database';
      FPortOption.Caption := 'Port';
  {$ENDIF}
    FSchemaOption.Caption := 'Schema';
  finally
    InSetLabelSet := False;
  end;
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.DefDialogClass: TClass;
begin
  Result := TPgConnectForm;
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetServerEnumeratorClass: TCRServerEnumeratorClass;
begin
  Result := TPgServerEnumerator;
end;

{$IFDEF MSWINDOWS}
function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetKeyPath: string;
begin
  Result := KeyPath;
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetServerStoreName: string;
begin
  Result := 'Server';
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SaveInfoToRegistry(Registry: TRegistry);
begin
  inherited;

  if Port.Visible then
    Registry.WriteInteger('Port', TPgConnection(Connection).Port)
  else
    Registry.DeleteValue('Port');

  if Database.Visible then
    Registry.WriteString('Database', TPgConnection(Connection).Database)
  else
    Registry.DeleteValue('Database');

  if Schema.Visible then
    Registry.WriteString('Schema', TPgConnection(Connection).Schema)
  else
    Registry.DeleteValue('Schema');
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.LoadInfoFromRegistry(Registry: TRegistry);
begin
  inherited;

  if Port.Visible and Registry.ValueExists('Port') then
    TPgConnection(Connection).Port := Registry.ReadInteger('Port');

  if Database.Visible and Registry.ValueExists('Database') then
    TPgConnection(Connection).Database := Registry.ReadString('Database');

  if Schema.Visible and Registry.ValueExists('Schema') then
    TPgConnection(Connection).Schema := Registry.ReadString('Schema');
end;
{$ENDIF}

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetConnection: TPgConnection;
begin
  Result := (inherited Connection) as TPgConnection;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetOptions(var Options: TConnectDialogOptionArray; Ordered: Boolean = True);
begin
  SetLength(Options, 6);
  Options[0] := UserName;
  Options[1] := Password;
  Options[2] := Server;
  Options[3] := Database;
  Options[4] := Port;
  Options[5] := Schema;

  inherited GetOptions(Options, Ordered);
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetDatabaseOption(Value: TConnectDialogOption);
begin
  FDatabaseOption.Assign(Value);
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetPortOption(Value: TConnectDialogOption);
begin
  FPortOption.Assign(Value);
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetSchemaOption(Value: TConnectDialogOption);
begin
  FSchemaOption.Assign(Value);
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetDatabaseLabel: string;
begin
  Result := FDatabaseOption.Caption;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetDatabaseLabel(Value: string);
begin
  FDatabaseOption.Caption := Value;
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetPortLabel: string;
begin
  Result := FPortOption.Caption;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetPortLabel(Value: string);
begin
  FPortOption.Caption := Value;
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetShowDatabase: Boolean;
begin
  Result := FDatabaseOption.Visible;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetShowDatabase(Value: Boolean);
begin
  FDatabaseOption.Visible := Value;
end;

function {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.GetShowPort: Boolean;
begin
  Result := FPortOption.Visible;
end;

procedure {$IFDEF FMX}TPgConnectDialogFmx{$ELSE}TPgConnectDialog{$ENDIF}.SetShowPort(Value: Boolean);
begin
  FPortOption.Visible := Value;
end;

initialization
{$IFDEF FMX}
  if not Assigned(DefConnectDialogClassProc) then
{$ENDIF}
    DefConnectDialogClassProc := DefConnectDialogClass;

{$IFDEF VER16P}
{$IFDEF FMX}
  GroupDescendentsWith(TPgConnectDialogFmx, TFmxObject);
{$ELSE}
  GroupDescendentsWith(TPgConnectDialog, TControl);
{$ENDIF}
{$ENDIF}

end.
